/**
 * @file    keys.c
 * @brief   Collection of key-management-routines
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/

/*
 *  Includes ---------------------------------------------------------------- 
 */
#include "keys.h"

/*
 *  Definitions -------------------------------------------------------------
 */
/** @brief Bitmask for pressed key */
#define KEY_DOWN        (1<<4)
/** @brief Bitmask for changed keystatus */
#define KEY_CHANGE      (1<<5)
/** @brief Bitmask for 100ms clock */
#define KEY_100MS_FLAG  (1<<6)
/** @brief Flag saves last state of key */
#define KEY_MEM         (1<<7)

/*
 *  Decalares --------------------------------------------------------------- 
 */
static uint8_t u8_key_timer[KEY_TOTAL_NUMBER]={0};
static uint8_t u8_key_flags[KEY_TOTAL_NUMBER]={0};
static uint8_t u8_key_duration[KEY_TOTAL_NUMBER]={0};

/*
 *  Prototypes --------------------------------------------------------------
 */
static uint32_t KEY_get_key_status(void);


/*
 *  Functions ---------------------------------------------------------------
 */
/*!
 ****************************************************************************
 * @brief Checks if the key is pressed
 * @param u8_key Number of key
 * @return Key is pressed (true/false)
**/ 
bool_t KEY_is_key_pressed( uint8_t u8_key )
{
  return (bool_t)( u8_key_flags[u8_key] & KEY_DOWN );
}


/*!
 ****************************************************************************
 * @brief Checks if the key is just pressed
 * @param u8_key Number of key
 * @return Key is just pressed (true/false)
**/ 
bool_t KEY_is_key_just_pressed( uint8_t u8_key )
{
  return (bool_t)( ( u8_key_flags[u8_key] & KEY_CHANGE )
              && ( u8_key_flags[u8_key] & KEY_DOWN ) );
}


/*!
 ****************************************************************************
 * @brief Checks if key is pressed for less than a time
 * @param u8_key Number of key
 * @param u8_maximum_time Time in x/100ms
 * @return Key is pressed for less than the commited time (true/false)
**/ 
bool_t KEY_is_key_pressed_short( uint8_t u8_key, uint8_t u8_maximum_time )
{
  return (bool_t)( ((u8_key_flags[u8_key] & (KEY_DOWN | KEY_CHANGE)) == KEY_DOWN)
               && (u8_key_timer[u8_key] < u8_maximum_time) );
}


/*!
 ****************************************************************************
 * @brief Checks if the key is pressed for a minimun of time
 * @param u8_key Number of key
 * @param u8_minimum_time Time in x/100ms
 * @return Key is pressed for minimum of time (true/false)
**/ 
bool_t KEY_is_key_pressed_for_minimum_time( uint8_t u8_key,
                                          uint8_t u8_minimum_time )
{
  return (bool_t)( ((u8_key_flags[u8_key] & (KEY_DOWN | KEY_CHANGE)) == KEY_DOWN) 
               && (u8_key_timer[u8_key] >= u8_minimum_time) );
}


/*!
 ****************************************************************************
 * @brief Checks if the key is pressed for a minimum
 * or a additional repeatable delay time
 * @param u8_key Number of key
 * @param u8_minimum_time Time in x/100ms
 * @param u8_delay_time Time in x/10ms
 * @return Key is pressed for minimum of time or repeated (true/false)
**/ 
bool_t KEY_is_key_pressed_for_minimum_time_with_repeat( uint8_t u8_key, 
                                                        uint8_t u8_minimum_time, 
                                                        uint8_t u8_delay_time )
{
  static uint8_t u8_local_delay[KEY_TOTAL_NUMBER] = {0};
  
  if( (u8_key_flags[u8_key] & (KEY_DOWN | KEY_CHANGE)) == (KEY_DOWN) 
     && (u8_key_timer[u8_key] >= u8_minimum_time) )
  {
    if( u8_local_delay[u8_key] )
    {
      u8_local_delay[u8_key]--;
    }
    if( u8_local_delay[u8_key] == 0 )
    {
      u8_local_delay[u8_key] = u8_delay_time;
      return (bool_t)1;
    }
    else
    {
      return (bool_t)0;
    }
  }
  else
  {
    u8_local_delay[u8_key] = 0;
    return (bool_t)0;
  }
}


/*!
 ****************************************************************************
 * @brief Checks if key is pressed exactly for a time
 * @param u8_key Number of key
 * @param u8_time Time in x/100ms
 * @return Key is pressed for the time (true/false)
**/ 
bool_t KEY_is_key_pressed_long( uint8_t u8_key, uint8_t u8_time )
{
  return (bool_t)( ((u8_key_flags[u8_key] & (KEY_DOWN | KEY_CHANGE)) == KEY_DOWN)
               && (u8_key_flags[u8_key] & KEY_100MS_FLAG)
               && (u8_key_timer[u8_key] == u8_time) );
}


/*!
 ****************************************************************************
 * @brief Checks if the key is just released
 * @param u8_key Number of key
 * @return Key is just released (true/false)
**/ 
bool_t KEY_is_key_just_released( uint8_t u8_key )
{
  return (bool_t)( (u8_key_flags[u8_key] & KEY_CHANGE)
             && ((u8_key_flags[u8_key] ^ KEY_DOWN) & KEY_DOWN) );
}


/*!
 ****************************************************************************
 * @brief Checks if key is released for less than a time
 * @param u8_key Number of key
 * @param u8_maximum_time Time in x/100ms
 * @return Key is released for less than the commited time (true/false)
**/ 
bool_t KEY_is_key_released_short( uint8_t u8_key, uint8_t u8_maximum_time )
{
  return (bool_t)( ((u8_key_flags[u8_key] & (KEY_DOWN | KEY_CHANGE)) == 0x00)
               && (u8_key_timer[u8_key] < u8_maximum_time) );
}


/*!
 ****************************************************************************
 * @brief Checks if the key is released for a minimun of time
 * @param u8_key Number of key
 * @param u8_minimum_time Time in x/100ms
 * @return Key is released for minimum of time (true/false)
**/ 
bool_t KEY_is_key_released_for_minimum_time( uint8_t u8_key, 
                                           uint8_t u8_minimum_time )
{
  return (bool_t)( ((u8_key_flags[u8_key] & (KEY_DOWN | KEY_CHANGE)) == 0x00) 
               && (u8_key_timer[u8_key] >= u8_minimum_time) );
}


/*!
 ****************************************************************************
 * @brief Checks if key is released exactly for a time
 * @param u8_key Number of key
 * @param u8_time Time in x/100ms
 * @return Key is released for the time (true/false)
**/ 
bool_t KEY_is_key_released_long( uint8_t u8_key, uint8_t u8_time )
{
  return (bool_t)( ((u8_key_flags[u8_key] & (KEY_DOWN | KEY_CHANGE)) == 0x00)
               && (u8_key_flags[u8_key] & KEY_100MS_FLAG)
               && (u8_key_timer[u8_key] == u8_time) );
}


/*!
 ****************************************************************************
 * @brief Checks if the key was released before the given time
 * @param u8_key Number of key
 * @param u8_time Time in x/100ms
 * @return Key was released before the given time (true/false)
**/ 
bool_t KEY_was_key_released_before_time( uint8_t u8_key, 
                                       uint8_t u8_time )
{
  return (bool_t)( (u8_key_flags[u8_key] & KEY_CHANGE)
             && ((u8_key_flags[u8_key] ^ KEY_DOWN) & KEY_DOWN)
             && ( u8_key_duration[u8_key] < u8_time) );
}


/*!
 ****************************************************************************
 * @brief Checks if the key was released after the given time
 * @param u8_key Number of key
 * @param u8_time Time in x/100ms
 * @return Key was released after the given time (true/false)
**/ 
bool_t KEY_was_key_released_after_time( uint8_t u8_key, 
                                      uint8_t u8_time )
{
  return (bool_t)( (u8_key_flags[u8_key] & KEY_CHANGE)
             && ((u8_key_flags[u8_key] ^ KEY_DOWN) & KEY_DOWN)
             && ( u8_key_duration[u8_key] >= u8_time) );
}


/*!
 ****************************************************************************
 * @brief Gives back the duration of the last stable keystatus
 * @param u8_key Number of key
 * @return Duration of the last stable keystatus
**/ 
uint8_t KEY_get_key_status_duration( uint8_t u8_key )
{
  return (u8_key_duration[u8_key]);
}


/*!
 ****************************************************************************
 * @brief   Checks the status of the defined keys
 * @details This function has to be called every 10ms!
**/ 
void KEY_do_keys( void )
{
  uint8_t u8_i;
  uint32_t u32_key_status = 0;
  
  u32_key_status = KEY_get_key_status();
  
  for(u8_i=0; u8_i < KEY_TOTAL_NUMBER; u8_i++)
  {
    // clear "one time" flags; flags may be set later in this function
    u8_key_flags[u8_i] &= ~(KEY_100MS_FLAG|KEY_CHANGE);

    // --- prescaled timer
    // 4 bit prescaler running in lower nibble of u8_key_flags
    if( (u8_key_flags[u8_i] & 0x0f) < 9 )
    {
      u8_key_flags[u8_i]++;                 // every 10 ms
    }
    else
    {
      u8_key_flags[u8_i] &= ~0x0f;          // clear prescaler
      if( u8_key_timer[u8_i] != 0xff )
      {
        u8_key_timer[u8_i]++;               // every 100 ms, if prescaler is not cleared in between
        u8_key_flags[u8_i] |= KEY_100MS_FLAG; // mark overflow
      }
    }
    // ----------

    // Key pressed?
    if((u32_key_status & ( 1UL << u8_i)))
    { // key is pressed
      if( !(u8_key_flags[u8_i] & KEY_MEM) )
      { // u8_key_status changed; debounce (re)start
        u8_key_flags[u8_i] |= KEY_MEM;
        u8_key_flags[u8_i] &= ~0x0f;        // clear prescaler
      }
      else if( (u8_key_flags[u8_i] & 0x0f) == KEY_DEBOUNCE_SAMPLES )
      { // Key pressed and debounced
        // Was the key just pressed or was it pressed before?
        if(!(u8_key_flags[u8_i] & KEY_DOWN))
        {
          // Key just pressed
          u8_key_flags[u8_i] |= KEY_CHANGE;
          u8_key_duration[u8_i] = u8_key_timer[u8_i];
          u8_key_timer[u8_i] = 0x00;
        }
        u8_key_flags[u8_i] |= KEY_DOWN;
      }	// end key is pressed and debounced
    } // end key is pressed
    else
    { // key is not pressed
      if( u8_key_flags[u8_i] & KEY_MEM )
      { // u8_key_status changed
        u8_key_flags[u8_i] &= ~KEY_MEM;
        u8_key_flags[u8_i] &= ~0x0f;        // clear prescaler
        // key was pressed before?
        if(u8_key_flags[u8_i] & KEY_DOWN)
        {
          u8_key_flags[u8_i] |= KEY_CHANGE;
          u8_key_duration[u8_i] = u8_key_timer[u8_i];
          u8_key_timer[u8_i] = 0x00;
        }
        u8_key_flags[u8_i] &= ~KEY_DOWN;
      }
    } // end key is not pressed
  } // end for
}

/*!
 ****************************************************************************
 * @brief Reads the status of the keys
 * @return Pressed keys in 32bit bitcoded value
**/ 
static uint32_t KEY_get_key_status( void )
{
  uint32_t u32_key_status = 0;
  if( GPIO_IDR(S1_PORT) & S1_PIN )
  {
    u32_key_status |= (1UL << S1);
  }
  
  if( GPIO_IDR(S2_PORT) & S2_PIN )
  {
    u32_key_status |= (1UL << S2);
  }
  
  if( GPIO_IDR(S3_PORT) & S3_PIN )
  {
    u32_key_status |= (1UL << S3);
  }
  
  if( GPIO_IDR(S4_PORT) & S4_PIN )
  {
    u32_key_status |= (1UL << S4);
  }
  
  if( GPIO_IDR(S5_PORT) & S5_PIN )
  {
    u32_key_status |= (1UL << S5);
  }
  
  if( GPIO_IDR(S6_PORT) & S6_PIN )
  {
    u32_key_status |= (1UL << S6);
  }
  
  if( GPIO_IDR(S7_PORT) & S7_PIN )
  {
    u32_key_status |= (1UL << S7);
  }
  
  if( GPIO_IDR(S8_PORT) & S8_PIN )
  {
    u32_key_status |= (1UL << S8);
  }
  
  if( GPIO_IDR(S9_PORT) & S9_PIN )
  {
    u32_key_status |= (1UL << S9);
  }
  
  if( GPIO_IDR(S10_PORT) & S10_PIN )
  {
    u32_key_status |= (1UL << S10);
  }
  
  if( GPIO_IDR(OUT1_PORT) & OUT1_PIN )
  {
    u32_key_status |= (1UL << OUT1);
  }
  
  if( GPIO_IDR(OUT2_PORT) & OUT2_PIN )
  {
    u32_key_status |= (1UL << OUT2);
  }
  
  if( GPIO_IDR(OUT3_PORT) & OUT3_PIN )
  {
    u32_key_status |= (1UL << OUT3);
  }
  
  if( GPIO_IDR(OUT4_PORT) & OUT4_PIN )
  {
    u32_key_status |= (1UL << OUT4);
  }
  
  if( GPIO_IDR(OUT5_PORT) & OUT5_PIN )
  {
    u32_key_status |= (1UL << OUT5);
  }
  
  if( GPIO_IDR(OUT6_PORT) & OUT6_PIN )
  {
    u32_key_status |= (1UL << OUT6);
  }
  
  if( GPIO_IDR(OUT7_PORT) & OUT7_PIN )
  {
    u32_key_status |= (1UL << OUT7);
  }
  
  if( GPIO_IDR(OUT8_PORT) & OUT8_PIN )
  {
    u32_key_status |= (1UL << OUT8);
  }
  
  u32_key_status = u32_key_status ^ 0xFFFFFFFF;
  return (u32_key_status);
}
/** @} */
/** @} */
/** @}
 * End of file
**/
