/**
 * @file    main.c
 * @brief   ELV-SH-IRS8 Firmware
 * @note    uses a modified version of IRMP - infrared multi-protocol decoder, supports several remote control protocols
 *          Copyright (c) 2009-2020 Frank Meyer - frank(at)fli4l.de
 *          GPL 2 (or later)
 * @note    uses libstm8 project.
 *          Copyright (C) 2015 Nikolay Merinov <nikolay.merinov@member.fsf.org>
 *          GPL 3 (or later)
 *
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/

/*
 *  Includes ------------------------------------------------------------------
 */
#include <stdio.h>
#include <libstm8/stm8l.h>
#include <libstm8/l/gpio.h>
#include "defines.h"
#include "config.h"
#include "timer.h"
#include "keys.h"
#include "my_functions.h"
#include "intrinsics.h"

/*
 *  Functions -----------------------------------------------------------------
 */
void main( void )
{
  CONFIG_clock( );
  /* Init general purpose in- and outputs */
  CONFIG_gpio( );

  CONFIG_usart( );
  
  /* Print header-information */
  DEBUG_message( "-------------------------------------------------------\r\n" );
  DEBUG_message( "Programmstart: ELV-SH-IRS8\r\n" );
  DEBUG_message( "Firmwareversion: %u.%u.%03u\r\n", 
                (uint16_t)VERSION_MAJOR_VERSION,
                (uint16_t)VERSION_MINOR_VERSION,
                (uint16_t)VERSION_BUILD_VERSION );
  DEBUG_message( "-------------------------------------------------------\r\n" );
  
  CONFIG_timer();
 
  /* Check if EEPROM is initialized */
  if( MY_FUNCTIONS_EEPROM_get_init_value( ) != EEPROM_INIT_VALUE )
  {
    DEBUG_message("\r\nEEPROM INIT\r\n" );
    MY_FUNCTIONS_EEPROM_init( );
  }
    
  MY_FUNCTIONS_IR_init();
  MY_FUNCTIONS_TIMER_LEDS_init();
  
  //Set ITC priority for TIMER2 and TIMER4
  VECTxSPR_REG(19) &= ~(VECTxSPR_MASK << VECTxSPR_SHIFT(19));
  VECTxSPR_REG(19) |= (ITC_SPR_2 << VECTxSPR_SHIFT(19));

  VECTxSPR_REG(25) &= ~(VECTxSPR_MASK << VECTxSPR_SHIFT(25));
  VECTxSPR_REG(25) |= (ITC_SPR_1 << VECTxSPR_SHIFT(25));


  /* Enable general interrupts */
  __enable_interrupt();
  
  TIMER_LED1_ON;
  TIMER_LED2_ON;
  GPIO_ODR(LED_RED_PORT) |= (LED_RED_PIN);
  Delay_ms(1000);
  GPIO_ODR(LED_GREEN_PORT) |= (LED_GREEN_PIN);
  Delay_ms(1000);
  GPIO_ODR(LED_RED_PORT) &= (uint8_t)(~LED_RED_PIN);
  Delay_ms(1000);
  GPIO_ODR(LED_GREEN_PORT) &= (uint8_t)(~LED_GREEN_PIN);
  TIMER_LED1_OFF;
  TIMER_LED2_OFF;
  
  while( 1 )
  {
    MY_FUNCTIONS_IR_do();

    if (TIMER_is_time_flag_set(TIMER_1MS))
    {
      /* Clear timer flag */
      TIMER_reset_time_flag(TIMER_1MS);
    }
    
    if (TIMER_is_time_flag_set(TIMER_10MS))
    {
      /** Clear timer flag */
      TIMER_reset_time_flag(TIMER_10MS);
      MY_FUNCTIONS_IR_timer();
      MY_FUNCTIONS_TIMER_LEDS_fading();
      KEY_do_keys();
      
      //long keypress of both timer buttons to start IR learning
      if( KEY_is_key_pressed_long( S9, 50 ) && KEY_is_key_pressed( S10 ) )
      {
        if(MY_FUNCTIONS_IR_get_state() == LEARNING_OFF)
        {
          MY_FUNCTIONS_IR_start_learning( 0xff );
          MY_FUNCTIONS_LED_set(LED_ORANGE, __TRUE);
        }
        else if(MY_FUNCTIONS_IR_get_state() == LEARNING_SELECT_CH)
        {
          
          MY_FUNCTIONS_EEPROM_set_init_value( FACTORY_RESET_VALUE );
          DEBUG_message("\r\nFactory Reset!\r\n" );
          // Reset the CPU: Enable the watchdog and wait until it expires
          WWDG_CR = 0x80u;
          while ( 1 );    // Wait until reset occurs from WWDG
        }
  
      }
      else
        //long keypress of timer buttons to enable/disable timers
      if( KEY_was_key_released_after_time( S9, 20) && KEY_was_key_released_before_time( S9, 50 ) && !KEY_is_key_pressed( S10 ) )
      {
        if( MY_FUNCTIONS_TIMER_tim1_is_active() )
        {
          MY_FUNCTIONS_TIMER_tim1_stop();
          TIMER_LED1_OFF;
        }
        else
        {
          MY_FUNCTIONS_TIMER_tim1_restart();
          TIMER_LED1_ON;
        }
      }
      else if( KEY_was_key_released_after_time( S10, 20) && KEY_was_key_released_before_time( S10, 50 ) && !KEY_is_key_pressed( S9 )  )
      {
        if( MY_FUNCTIONS_TIMER_tim2_is_active() )
        {
          MY_FUNCTIONS_TIMER_tim2_stop();
          TIMER_LED2_OFF;
        }
        else
        {
          MY_FUNCTIONS_TIMER_tim2_restart();
          TIMER_LED2_ON;
        }
      }
      
      //short keypress of channel and timer button
      for(uint8_t i=S1; i<= S10; i++)
      {
        if( KEY_was_key_released_before_time( i, 15 ) )
        {
          if(MY_FUNCTIONS_IR_get_state() == IR_SEND)
          {
            MY_FUNCTIONS_IR_abort_sending();
          }
          else if(MY_FUNCTIONS_IR_get_state() == LEARNING_SELECT_CH)
          {
            MY_FUNCTIONS_IR_start_learning( i );
            MY_FUNCTIONS_LED_set(LED_PULSE_ORANGE, __TRUE);
          }
          else if(MY_FUNCTIONS_IR_get_state() == LEARNING_ON)
          {
              MY_FUNCTIONS_IR_stop_learning(); //stop learning or start off commands for timer channels
          }
          else if(MY_FUNCTIONS_IR_get_state() == LEARNING_OFF)
          {
            //manual switching for timer channels
            if(i == S9)
            {
              if( MY_FUNCTIONS_TIMER_tim1_get_laststate() != __TRUE )
              {
                MY_FUNCTIONS_IR_send( S9 );
                MY_FUNCTIONS_TIMER_tim1_set_laststate( __TRUE );
                if( MY_FUNCTIONS_TIMER_tim1_is_active() )
                {
                   TIMER_LED1_ON;
                }
                else
                {
                  TIMER_LED1_LOWON;
                }
              }
              else
              {
                MY_FUNCTIONS_IR_send( S9+2 );
                MY_FUNCTIONS_TIMER_tim1_set_laststate( __FALSE );
                if( MY_FUNCTIONS_TIMER_tim1_is_active() )
                {
                   TIMER_LED1_FADING;
                }
                else
                {
                  TIMER_LED1_OFF;
                }
              }
            }
            else if(i == S10)
            {
              if( MY_FUNCTIONS_TIMER_tim2_get_laststate() != __TRUE )
              {
                MY_FUNCTIONS_IR_send( S10 );
                MY_FUNCTIONS_TIMER_tim2_set_laststate( __TRUE );
                if( MY_FUNCTIONS_TIMER_tim2_is_active() )
                {
                   TIMER_LED2_ON;
                }
                else
                {
                  TIMER_LED2_LOWON;
                }
              }
              else
              {
                MY_FUNCTIONS_IR_send( S10+2 );
                MY_FUNCTIONS_TIMER_tim2_set_laststate( __FALSE );
                if( MY_FUNCTIONS_TIMER_tim2_is_active() )
                {
                   TIMER_LED2_FADING;
                }
                else
                {
                  TIMER_LED2_OFF;
                }
              }
            }
            //manual switching for other channels
            else
            {
              MY_FUNCTIONS_IR_send( i );
            }
          }
        }
      }
      
      //check inputs of HmIP-MOD-OC8
      for(uint8_t i=OUT1; i<= OUT8; i++)
      {
//        if( KEY_was_key_released_before_time( i, 15 ) )
        if( KEY_is_key_just_pressed( i ) || KEY_is_key_just_released( i ) )
        {
          MY_FUNCTIONS_IR_send( i-OUT1 );
        }
      }
    }
    
    if (TIMER_is_time_flag_set(TIMER_100MS))
    {
      /* Clear timer flag */
      TIMER_reset_time_flag(TIMER_100MS);
      MY_FUNCTIONS_LED_do();
    }
    
    if (TIMER_is_time_flag_set(TIMER_1S))
    {
      /* Clear timer flag */
      TIMER_reset_time_flag(TIMER_1S);
      MY_FUNCTIONS_TIMER_isr();
    }
  }
}

/** 
 * \brief This function is required for the printf output
 * 
 * The putchar function writes the character c to the
 * output-stream pointed to by stream.
 * The function returns the character written.
 * 
 * The putchar function must be adapted to the
 * particular features of the hardware.
 * 
 * \param c The character that is to be written
 * 
 * \return The written character
 * 
**/ 
int putchar(int c) {
  // Write a character to the USART
  usart_send(USART1, c);
  // Loop until the end of transmission
  usart_wait_send_ready(USART1);
  
  return (c);
}
/** @}
 * End of file
**/
