/**
 * @file    timer.c
 * @brief   This file provides all the routines for timer-handling and delays
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/

/*
 * Includes section ---------------------------------------------------------
 */
#include "typedefs_stm8.h"
#include "defines.h"
#include "timer.h"

/**
 *  Private Variables --------------------------------------------------------
**/
static volatile uint8_t u8_timer_flags = 0;

/*
 *  Functions ---------------------------------------------------------------
 */
/*!
 ****************************************************************************
 * @brief   Checks if the given time flag is set
 * @param   time_flag_t Time flag to check
 * @return  0 = Time flag is not set
 *          1 = Time flag is set
**/
uint8_t TIMER_is_time_flag_set(time_flag_t time_flag)
{
  return ((u8_timer_flags & time_flag) != 0);
}


/*!
 ****************************************************************************
 * @brief   Resets the given time flag
 * @param   time_flag_t Time flag to clear
 * @return  0 = Time flag is not set
 *          1 = Time flag is set
**/
void TIMER_reset_time_flag(time_flag_t time_flag)
{
  u8_timer_flags &= ~time_flag;
}


/*!
 ****************************************************************************
 * @brief   Sets the timerflags
 * @details This function has to be called every 1 ms!
**/
void TIMER_do_timer(void)
{
  static uint16_t u16_timer_tag = 0;
  
  ++u16_timer_tag;
  
  u8_timer_flags |= TIMER_1MS;

  if((u16_timer_tag%10) == 0)
  {
    u8_timer_flags |= TIMER_10MS;
    
    if((u16_timer_tag%100) == 0)
    {
      u8_timer_flags |= TIMER_100MS;
      
      if(u16_timer_tag>=1000)
      {
        u8_timer_flags |= TIMER_1S;
        u16_timer_tag=0;
      }
    }
  }
}

/*
  * @brief geenrates delays using timer module
  * @param  uint16_t delay -  delay in milli seconds
  * @retval None
  */
void Delay_ms(uint16_t delay)
{
  for(uint16_t i = 0; i < delay;)
  {
    if (TIMER_is_time_flag_set(TIMER_1MS))
    {
      /* Clear timer flag */
      TIMER_reset_time_flag(TIMER_1MS);
      i++;
    }
  }
}
/** @}
 * End of file
**/

